var generalApiModule = require('./generalApiModule');
var _ = require('underscore');

var SUPPORTEDPLATFORMS = generalApiModule.SUPPORTEDPLATFORMS;
var CONSTANTS = generalApiModule.CONSTANTS;
var GeneralAPI = generalApiModule.GeneralAPI;


var FacebookAPI = function(bonoboToken, fToken = ""){

    var facebookAPI = {};
    facebookAPI.__proto__ = GeneralAPI(SUPPORTEDPLATFORMS.facebook, bonoboToken);
    var faceToken = fToken;

    facebookAPI.updateFaceToken = function updateFaceToken(newToken){
      faceToken = newToken;
    };

    // Sends user data to bonobo (as it arrives from facebook)
    function sendUserToBonobo(jsonData, timeDate, intentsList){
      var dataToSend = facebookAPI.getBasicJson(jsonData, timeDate, intentsList);
      var content = facebookAPI.sendToBonoboWithPOSTRequest(dataToSend);
      return content;
    };

    // Sends bot data to bonobo in a specific format according to bonobo api
    function sendBotToBonobo(jsonData, timeDate, intentsList){
      var dataToSend = facebookAPI.getBasicJson(jsonData, timeDate, intentsList);
      if (dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.JSON] === undefined){
        dataToSend[CONSTANTS.FULL_JSON] = {
          "json" : jsonData
        }
        dataToSend = facebookAPI.addOptionalParams(dataToSend, timeDate, intentsList);
      }
      if (dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.QS] === undefined){
        dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.QS] = {
          "access_token": faceToken
        }
      }
      else{
        if (dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.QS][CONSTANTS.ACCESSTOKEN] !== undefined &&
			dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.QS][CONSTANTS.ACCESSTOKEN] != faceToken &&
            dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.QS][CONSTANTS.ACCESSTOKEN] != ""){
          faceToken = dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.QS][CONSTANTS.ACCESSTOKEN];
        }
      }
      var content = facebookAPI.sendToBonoboWithPOSTRequest(dataToSend);
      return content;
    };

    facebookAPI.sendDataToBonobo = function sendDataToBonobo(jsonData, timeDate = undefined, intentsList = undefined){
      generalApiModule.assertDictInput(jsonData);
      if (jsonData.entry !== undefined){
        return (sendUserToBonobo(jsonData, timeDate, intentsList));
      }
      else{
        return (sendBotToBonobo(jsonData, timeDate, intentsList));
      }
    };

    facebookAPI.receive = function receive(bot, message, next){
      if (message.is_echo === undefined){
        var messageClone = _.clone(message);
        var user_id = messageClone.user;
        var timeS = message.timestamp;
        delete messageClone.user;
        delete messageClone.timestamp;
        delete messageClone.channel;
        var new_json =
        {
           "object": "page",
           "entry": [
                  {
                   "id": user_id,
                   "time": timeS,
                   "messaging": [
                           {
                             "sender": {
                               "id": user_id
                              },
                             "timestamp": timeS,
                             "message": messageClone
                           }
                         ]
                }
           ]
        };
        sendUserToBonobo(new_json, undefined);

      }
      next();
    };

    facebookAPI.send = function send(bot, message, next){

      var messageClone = _.clone(message);
      var channel = messageClone.channel;
      delete messageClone.channel;
      var new_json =
      {
    		qs: {access_token: bot.botkit.config.access_token},
    		method: 'POST',
    		json: {
    			recipient: {id:channel},
    			message: messageClone,
    		}
    	};
      sendUserToBonobo(new_json, new Date().getTime());
      next();
    };

    return facebookAPI;
};

// exports module's class
module.exports.FacebookAPI = FacebookAPI;
