
const request = require('request');
const util = require('util');

const SUPPORTEDPLATFORMS = {
  generic : "generic",
  viber: "viber",
  slack: "slack",
  facebook : "facebook"
};

const CONSTANTS = {
  BONOBOAPIADRESS : "http://api.bonobo.ai/apiv1/addmessage/",
  FULL_JSON : "full_json",
  PLATFORM : "platform",
  TOKEN : "token",
  INTENTS_LIST : "intents_list",
  DATE_TIME : "date_time",
  JSON : "json",
  QS : "qs",
  ACCESSTOKEN : "access_token"
};

var assertDictInput = function (dataJson, input){
  if (typeof(dataJson) != typeof({})){
    throw Error(" dataJson should be a JSON Object");
  }
};

var GeneralAPI = function(platform = "", bonoboToken = ""){
    // General API with standard functions for all the different subclasses
    var bonoboToken = bonoboToken;
    var platform = platform;

    function updateBonoboToken(newToken) {
      bonoboToken = newToken;
    };

    // Returns basic structure of bonobo api json with the "jsonData" Data
    function getBasicJson(jsonData, timeDate = undefined, intentsList = undefined){

      var dataJson = {};
      dataJson[CONSTANTS.PLATFORM] = platform;
      dataJson[CONSTANTS.TOKEN] = bonoboToken;
      dataJson[CONSTANTS.FULL_JSON] = jsonData;

      dataJson = addOptionalParams(dataJson, timeDate, intentsList);

      return dataJson;
    };

    function sendToBonoboWithPOSTRequest(dataTosend){
      var options = {
        url: CONSTANTS.BONOBOAPIADRESS,
        method: 'POST',
        json: dataTosend
      };
      var content = request(options, function(error, response, body) {
        console.log(response +""+body);
        if (error) {
          throw Error("Error sending messages: ", error)
        }
      });
      return content;
    };

    // Returns intents list as an array for th json object
   function getIntentsList (intentsList){
      if (util.isArray(intentsList)){
        return intentsList;
      }
      else if (typeof(intentsList) === typeof("")){
        return [intentsList];
      }
      else{
        throw Error("intentsList needs to be a Array / String");
      }
    };

    // add optional parameters to to json object(dataToSend)
    function addOptionalParams(dataToSend, timeDate = undefined, intentsList = undefined){
      if (intentsList != undefined){
        dataToSend[CONSTANTS.INTENTS_LIST] = getIntentsList(intentsList);
      }
      if (timeDate != undefined){
        dataToSend[CONSTANTS.DATE_TIME] = timeDate;
      }
      return dataToSend;
    };

    return {
    bonoboToken: bonoboToken,
    platform: platform,
    getBasicJson: getBasicJson,
    updateBonoboToken: updateBonoboToken,
    sendToBonoboWithPOSTRequest: sendToBonoboWithPOSTRequest,
    getIntentsList: getIntentsList,
    addOptionalParams: addOptionalParams
  }

};

//exports module's class, functions and constants
module.exports.assertDictInput = assertDictInput;
module.exports.GeneralAPI = GeneralAPI;
module.exports.SUPPORTEDPLATFORMS = SUPPORTEDPLATFORMS;
module.exports.CONSTANTS = CONSTANTS;
