var generalApiModule = require('./generalApiModule');
const util = require('util');
const request = require('request');
const SUPPORTEDPLATFORMS = generalApiModule.SUPPORTEDPLATFORMS;
const CONSTANTS = generalApiModule.CONSTANTS;
const GeneralAPI = generalApiModule.GeneralAPI;

var GenericAPI = function(bonoboToken = ""){
    // General API with standard functions for all the different subclasses
    var platform = SUPPORTEDPLATFORMS["generic"];
    var genericAPI = {};
    var bonoboToken = bonoboToken;

    genericAPI.updateBonoboToken = function updateBonoboToken(newToken) {
      bonoboToken = newToken;
    };

    genericAPI.sendDataToBonobo = function sendDataToBonobo(text, channel_id, user_id, platform, who_speaks, date_time = undefined, intents_list = undefined){
      // input validation
      if (who_speaks != "bot" && who_speaks != "user"){
        throw Error("'who_speaks' parameter error: ('bot'/'user') MESSAGE WAS NOT SENT");
      }

      var dataJson = {};
      dataJson[CONSTANTS.PLATFORM] = platform;
      dataJson[CONSTANTS.TOKEN] = bonoboToken;
      dataJson["text"] = text;
      dataJson["who_speaks"] = who_speaks;
      dataJson["user_id"] = user_id;
      dataJson["id_channel"] = channel_id;
      if (date_time != undefined){
        dataJson[CONSTANTS.DATE_TIME] = date_time;
      }
      if (intents_list != undefined){
        intents_list = getIntentsList(intents_list);
        dataJson[CONSTANTS.INTENTS_LIST] = intents_list;
      }
      var content = sendToBonoboWithPOSTRequest(dataJson);
      return content;
    }

    function sendToBonoboWithPOSTRequest(dataTosend){
      var options = {
        url: CONSTANTS.BONOBOAPIADRESS,
        method: 'POST',
        json: dataTosend
      };
      var content = request(options, function(error, response, body) {
        if (error) {
          throw Error("Error sending messages: ", error);
        } else if (response.body.error) {
          throw Error("Error: ",  response.body.error);
        }
      });
      return content;
    };

	// Returns intents list as an array for th json object
	function getIntentsList (intentsList){
	  if (util.isArray(intentsList)){
		    return intentsList;
	  }
	  else if (typeof(intentsList) === typeof("")){
		    return [intentsList];
	  }
	  else{
		    throw Error("intentsList needs to be a Array / String");
	  }
	};

    return genericAPI;
};

// exports module's class
module.exports.GenericAPI = GenericAPI;
