var generalApiModule = require('./generalApiModule');
var _ = require('underscore');

var SUPPORTEDPLATFORMS = generalApiModule.SUPPORTEDPLATFORMS;
var CONSTANTS = generalApiModule.CONSTANTS;
var GeneralAPI = generalApiModule.GeneralAPI;

var SlackAPI = function(bonoboToken, sToken = ""){
    var slackAPI = {};
    slackAPI.__proto__ = GeneralAPI(SUPPORTEDPLATFORMS.slack, bonoboToken);
    var slackToken = sToken;

    slackAPI.updateSlackToken = function updateSlackToken(newToken){
      slackToken = newToken;
    };

    // Sends user data to bonobo (as it arrives from facebook)
    slackAPI.sendUserToBonobo = function sendUserToBonobo(jsonData, timeDate = undefined, intentsList = undefined){
      generalApiModule.assertDictInput(jsonData);
      var dataToSend = slackAPI.getBasicJson(jsonData, timeDate, intentsList);
      var content = slackAPI.sendToBonoboWithPOSTRequest(dataToSend);
      dataToSend = slackAPI.addOptionalParams(dataToSend, timeDate, intentsList);
      return content;
    };

    // Sends bot data to bonobo in a specific format according to bonobo api
    slackAPI.sendBotToBonobo = function sendBotToBonobo(jsonData, timeDate = undefined, intentsList = undefined){
      dataToSend = arrangeBotJsonToBonoboFormat(jsonData, timeDate, intentsList);
      var content = slackAPI.sendToBonoboWithPOSTRequest(dataToSend);
      return content;
    };

    //returns a json object in a format which bonobo accepts from a bot.
    function arrangeBotJsonToBonoboFormat(jsonData, timeDate = undefined, intentsList = undefined){
      generalApiModule.assertDictInput(jsonData);
      var dataToSend = {};
      dataToSend = slackAPI.getBasicJson("", timeDate, intentsList);
      if (jsonData["message"] === undefined){
        dataToSend[CONSTANTS.FULL_JSON] = {
          "message" : jsonData
        };
      }
      else{
        dataToSend[CONSTANTS.FULL_JSON] = jsonData;
      }

      dataToSend = slackAPI.addOptionalParams(dataToSend, timeDate, intentsList);

      if (dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.TOKEN] === undefined){
        dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.TOKEN] = slackToken;
      }
	  else{
		  if (dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.TOKEN] != slackToken &&
				dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.TOKEN] != ""){
					slackToken = dataToSend[CONSTANTS.FULL_JSON][CONSTANTS.TOKEN];
				}
	  }
      return dataToSend;
    };

    function getFirstJson (bot, message){
      var botClone = _.clone(bot);
      var id = botClone.identity;
      var teamInformation = botClone.team_info;
      delete id.prefs;
      delete teamInformation.prefs;

      return ({
        team: teamInformation,
        bot: id,
        token: bot.config.token,
        message: message
      });

    };

    slackAPI.receive = function receive(bot, message, next){
      //only message types and not echo
      if (message.type == 'message' && !message.bot_id){
        // send to bonobo only messages (not notification or connection messages)
        slackAPI.sendUserToBonobo(getFirstJson(bot,message));
      }
      next();
    };

    slackAPI.send = function send(bot, message, next){
      slackAPI.sendBotToBonobo(getFirstJson(bot,message), new Date().getTime());
      next();
    };

    return slackAPI;
};

// exports module's class
module.exports.SlackAPI = SlackAPI;
